package cordova.plugin.ubx.redscan;
import static com.ubx.usdk.util.ConvertUtils.bytesToHexString;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.device.scanner.configuration.PropertyID;
import android.device.scanner.configuration.Symbology;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Handler;
import android.os.Message;
import android.preference.CheckBoxPreference;
import android.preference.EditTextPreference;
import android.preference.ListPreference;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.Toast;
import android.device.ScanManager;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CallbackContext;
import org.apache.cordova.PluginResult;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

/**
 * This class echoes a string called from JavaScript.
 */
public class ubxredscan extends CordovaPlugin {
    private static final String TAG = "ScanManager";

    private static final String ACTION_DECODE = ScanManager.ACTION_DECODE;   // default action
    private static final String ACTION_DECODE_IMAGE_REQUEST = "action.scanner_capture_image";
    private static final String ACTION_CAPTURE_IMAGE = "scanner_capture_image_result";
    private static final String BARCODE_LENGTH_TAG = ScanManager.BARCODE_LENGTH_TAG;
    private static final String DECODE_DATA_TAG = ScanManager.DECODE_DATA_TAG;
    
    private ScanManager mScanManager = null;
    private static boolean mScanEnable = true;
    private static boolean mScanCaptureImageShow = false;

    private static final int[] SCAN_KEYCODE = {520, 521, 522, 523};
    private CallbackContext callbackContext;
    private BroadcastReceiver mReceiver = new BroadcastReceiver() {

        @Override
        public void onReceive(Context context, Intent intent) {
            try{
                String action = intent.getAction();
                LogI("onReceive , action:" + action);
                byte[] barcode = intent.getByteArrayExtra(DECODE_DATA_TAG);
                int barcodeLen = intent.getIntExtra(BARCODE_LENGTH_TAG, 0);
                if (mScanCaptureImageShow) {
                    context.sendBroadcast(new Intent(ACTION_DECODE_IMAGE_REQUEST));
                }
                String scanResult = new String(barcode, 0, barcodeLen);
                JSONObject content = new JSONObject();
                content.put("result", scanResult);
                PluginResult pluginResult = new PluginResult(PluginResult.Status.OK,content);
                pluginResult.setKeepCallback(false); // Keep callback
                callbackContext.sendPluginResult(pluginResult);
            } catch (JSONException e) {
                Log.w(TAG, e);
                callbackContext.error(e.getMessage());
            }

        }
    };


    @Override
    public boolean execute(String action, JSONArray args, CallbackContext callbackContext) throws JSONException {
        if (action.equals("initredscan")) {
            this.initScan(callbackContext);
            return true;
        }
        if (action.equals("startredscan")) {
            this.startDecode(callbackContext);
            return true;
        }
        if (action.equals("stopredscan")) {
            this.stopDecode(callbackContext);
            return true;
        }
        if (action.equals("closeScanner")) {
            this.closeScanner(callbackContext);
            return true;
        }
        if (action.equals("lockTrigger")) {
            this.lockTrigger(callbackContext);
            return true;
        }
        if (action.equals("unlockTrigger")) {
            this.unlockTrigger(callbackContext);
            return true;
        }
        if (action.equals("resetScannerParameters")) {
            this.resetScannerParameters(callbackContext);
            return true;
        }

        return false;
    }

    @Override
    public void onDestroy() {
        if (mScanManager != null) {
            mScanManager = null;
        }
        cordova.getContext().unregisterReceiver(mReceiver);
    }

    private void initScan(CallbackContext callbackContext) {
        JSONObject content = new JSONObject();
        try {
            mScanManager = new ScanManager();
            boolean powerOn = mScanManager.getScannerState();
            if (!powerOn) {
                powerOn = mScanManager.openScanner();

                if (!powerOn) {
                    content.put("result", "init fail");
                    PluginResult pluginResult = new PluginResult(PluginResult.Status.ERROR,content);
                    pluginResult.setKeepCallback(false); // Keep callback
                    callbackContext.sendPluginResult(pluginResult);
                    return;
                }
            }
            mScanManager.switchOutputMode(0);
            IntentFilter filter = new IntentFilter();
            int[] idbuf = new int[]{PropertyID.WEDGE_INTENT_ACTION_NAME, PropertyID.WEDGE_INTENT_DATA_STRING_TAG};
            String[] value_buf = mScanManager.getParameterString(idbuf);
            if (value_buf != null && value_buf[0] != null && !value_buf[0].equals("")) {
                filter.addAction(value_buf[0]);
            } else {
                filter.addAction(ACTION_DECODE);
            }
            filter.addAction(ACTION_CAPTURE_IMAGE);
            cordova.getContext().registerReceiver(mReceiver , filter);
            content.put("result", "init success");
            PluginResult pluginResult = new PluginResult(PluginResult.Status.OK,content);
            pluginResult.setKeepCallback(false); // Keep callback
            callbackContext.sendPluginResult(pluginResult);
        } catch (JSONException e) {
            Log.w(TAG, e);
            callbackContext.error(e.getMessage());
        }


    }

    private void startDecode(CallbackContext callbackContext) {
        if (!mScanEnable) {
            LogI("startDecode ignore, Scan enable:" + mScanEnable);
            return;
        }
        boolean lockState = getlockTriggerState();
        if (lockState) {
            LogI("startDecode ignore, Scan lockTrigger state:" + lockState);
            return;
        }
        if (mScanManager != null) {
            this.callbackContext = callbackContext;
            mScanManager.openScanner();
            mScanManager.startDecode();
        }
    }

    private void stopDecode(CallbackContext callbackContext) {
        if (!mScanEnable) {
            LogI("stopDecode ignore, Scan enable:" + mScanEnable);
            return;
        }
        if (mScanManager != null) {
            mScanManager.stopDecode();
            callbackContext.success("OK");
        }
    }

    private void closeScanner(CallbackContext callbackContext) {
        if (!mScanEnable) {
            LogI("stopDecode ignore, Scan enable:" + mScanEnable);
            return;
        }
        if (mScanManager != null) {
            mScanManager.closeScanner();
            callbackContext.success("OK");
        }
    }

    private void lockTrigger(CallbackContext callbackContext) {
        if (!mScanEnable) {
            LogI("stopDecode ignore, Scan enable:" + mScanEnable);
            return;
        }
        if (mScanManager != null) {
            mScanManager.lockTrigger();
            callbackContext.success("OK");
        }
    }

    private void unlockTrigger(CallbackContext callbackContext) {
        if (!mScanEnable) {
            LogI("stopDecode ignore, Scan enable:" + mScanEnable);
            return;
        }
        if (mScanManager != null) {
            mScanManager.unlockTrigger();
            callbackContext.success("OK");
        }
    }

    private void resetScannerParameters(CallbackContext callbackContext) {
        if (!mScanEnable) {
            LogI("stopDecode ignore, Scan enable:" + mScanEnable);
            return;
        }
        if (mScanManager != null) {
            mScanManager.resetScannerParameters();
            callbackContext.success("OK");
        }
    }

    private boolean getlockTriggerState() {
        boolean state = mScanManager.getTriggerLockState();
        return state;
    }

    private void LogI(String msg) {
        android.util.Log.i(TAG, msg);
    }

}
